@@ -1,4 +1,4 @@
-# Copyright (C) 2013 - 2019  Metrum Research Group, LLC
+# Copyright (C) 2013 - 2019  Metrum Research Group
 #
 # This file is part of mrgsolve.
 #
@@ -18,7 +18,7 @@
 valid_funs <- function(x) {
   x1 <- length(x)==4
   x2 <- identical(names(x), c("main", "ode", "table", "config"))
-  if(x1 & x2) return(list(TRUE,""))
+  if(x1 & x2) return(list(TRUE,NULL))
   msg <- c(
     "Invalid functions specification.",
     "This model object is not compatible with the current mrgsolve version.",
@@ -35,7 +35,6 @@ check_names <- function(x,par,cmt) {
   us <-  any(charthere(x,"_"))
   res <- any(is.element(x,Reserved))
   alph <- !all(grepl("^[[:alpha:]]",x))
-  
   ans <- character(0)
   
   ## Duplicate names are not allowed
@@ -67,14 +66,14 @@ check_us <- function(x,cmt,ans) {
   leading <- x[substr(x,1,1)=="_"]
   if(length(leading) > 0) {
     ans <- c(ans, 
-             paste0("Leading underscore not allowed: ", 
+             paste0("leading underscore not allowed: ", 
                     paste(leading, collapse=" "))) 
   }
   check <- as.character(sapply(c("F_", "ALAG_", "D_", "R_"),paste0,cmt))
   iv_name <- intersect(x,check)
   if(length(iv_name) > 0) {
     ans <- c(ans, 
-             paste0("Reserved symbols in model names: ", iv_name))
+             paste0("reserved symbols in model names: ", iv_name))
   } 
   return(ans)
 }
@@ -110,11 +109,11 @@ protomod <- list(model=character(0),
                  preclean=FALSE,
                  atol=1E-8,
                  rtol=1E-8,
-                 maxsteps=5000,
+                 maxsteps=20000,
                  hmin=0,
                  hmax=0,
                  ixpr=0,
-                 mxhnil=0,
+                 mxhnil=2,
                  shlib=list(date="",par="", cmt="", compiled=FALSE, 
                             version=NULL, source=""),
                  funs = c(main=character(0),
@@ -127,6 +126,10 @@ protomod <- list(model=character(0),
                  param = new("parameter_list"),
                  init=new("cmt_list"),
                  capture=character(0),
+                 Icap = integer(0),
+                 capL = character(0),
+                 Icmt = integer(0), 
+                 cmtL = character(0),
                  args = list(),
                  fixed  = list(),
                  advan=13,
@@ -151,15 +154,24 @@ valid.mrgmod <- function(object) {
   x <- check_names(tags,Pars(object),Cmt(object))
   x1 <- length(x)==0
   x2 <- object@advan %in% c(0,1,2,3,4,13)
+  x3 <- !any(object@capture %in% Cmt(object))
   fun <- valid_funs(object@funs)
-  cool <- x1 & x2 & fun[[1]]
+  cool <- x1 & x2 & fun[[1]] & x3
   if(cool) return(TRUE)
   x <- c(x,fun[[2]])
   if(!x2) x <- c(x,"advan must be 1, 2, 3, 4, or 13")
+  if(!x3) {
+    invalid <- intersect(object@capture,Cmt(object))
+    invalid <- paste0(invalid,collapse=",")
+    invalid <- paste0("compartment should not be in $CAPTURE: ", invalid)
+    x <- c(x,invalid)
+  }
   return(x)
 }
 # nocov end
 
+
+
 ##' S4 class for mrgsolve model object
 ##'
 ##' @section Notes:
@@ -215,6 +227,10 @@ valid.mrgmod <- function(object) {
 ##' @slot funs symbol names for model functions in the shared object
 ##' @slot annot model annotations \code{<list>}
 ##' @slot plugin model plugins \code{<character>}
+##' @slot Icap capture indices to recover in the simulation
+##' @slot capL labels for \code{Icap}
+##' @slot Icmt compartment indices to recover in the simulation
+##' @slot cmtL labels for \code{Icmt}
 ##' 
 ##' @seealso \code{\link[mrgsolve]{update}}, \code{\link{solversettings}}
 ##' @keywords internal
@@ -232,6 +248,17 @@ setClass("packmod",
          )
 )
 
+initialize_mrgmod <- function(.Object, ...) {
+  .Object <- callNextMethod()
+  .Object@shlib[["cmt"]] <- Cmt(.Object)
+  .Object@shlib[["par"]] <- Pars(.Object)
+  .Object@shlib[["neq"]] <- length(.Object@shlib[["cmt"]])
+  .Object@shlib[["version"]] <- GLOBALS[["version"]]
+  .Object <- default_outputs(.Object)
+  .Object
+}
+setMethod("initialize", "mrgmod", initialize_mrgmod)
+
 ##' Return a pre-compiled, PK/PD model
 ##' 
 ##' @param ... passed to \code{\link[mrgsolve]{update}}
@@ -326,7 +353,9 @@ Param_list <- function(x) x@param@data
 Pars <-  function(x) names(x@param@data)
 Init <- function(x) x@init
 Cmt <-  function(x) names(x@init@data)
-
+Cmti <- function(x) x@Icmt
+Capturei <- function(x) x@Icap
+CAPTUREI <- function(x) c(length(x@capture),x@Icap-1L)
 
 ##' Return the location of the model shared object
 ##'
@@ -416,6 +445,9 @@ setMethod("names", "mrgmod", function(x) {
 ##' - `delta`: simulation time step
 ##' - `add`: additional simulation times
 ##' - `capture`: names of captured data items
+##' - `request`: compartments requested upon simulation
+##' - `cmti`: named indices for current output compartments
+##' - `capturei`: named indices for current output capture
 ##' - `random`: names and labels of `$OMEGA` and `$SIGMA`
 ##' - `code`: model source code from `cfile`
 ##' - `details`: model details data frame
@@ -427,7 +459,7 @@ setMethod("names", "mrgmod", function(x) {
 ##' - `envir`: the model environment
 ##' - `plugins`: plugins invoked in the model
 ##' - `digits`: number of digits to request in simulated data
-##' - `request`: compartments requested upon simulation
+
 ##' - `tscale`: multiplicative scalar for time in results only
 ##' - `mindt`: simulation output time below which there model will assume to 
 ##'   have not advanced
@@ -463,6 +495,8 @@ setMethod("as.list", "mrgmod", function(x, deep = FALSE, ...) {
     details <- x@annot
     code <- x@code
     random <- names(x)[c("omega", "sigma", "omega_labels", "sigma_labels")]
+    out_cmt <- x@cmtL
+    out_cap <- x@capL
     request <- x@request
     capture <- x@capture
     add <- x@add
@@ -481,6 +515,7 @@ setMethod("as.list", "mrgmod", function(x, deep = FALSE, ...) {
     init <- as.list(init(x))
     param <- as.list(param(x))
     cmt <- cmt(x)
+    
     covariates <- as.character(x@shlib$covariates)
     pars <- pars(x)
     neq <- neq(x)
@@ -617,15 +652,18 @@ summary.mrgmod <- function(object,...) {
   l <- as.list(object)
   ncmt <- l[["neq"]]
   npar <- l[["npar"]]
-  message("Model: ", l$model)
-  message("- Parameters: [", npar, "]")
-  message(prvec(l$pars,width = 50,prefix="  "))
-  message("- Compartments: [", ncmt, "]")
-  message(prvec(l$cmt,width = 50, prefix = "  "))
-  message("- Captured: [", length(l[["capture"]]), "]")
+  cat("Model: ", l$model, "\n",sep="")
+  cat("- Parameters: [", npar, "]", "\n",sep="")
+  cat(prvec(l$pars,width = 50,prefix="  "), "\n",sep="")
+  cat("- Compartments: [", ncmt, "]", "\n",sep="")
+  cat(prvec(l$cmt,width = 50, prefix = "  "), "\n",sep="")
+  cat("- Captured: [", length(l[["capture"]]), "]", "\n",sep="")
   o <- l$capture
   if(length(o)==0) o <- "<none>"
-  message(prvec(o, width = 50, prefix = "  "))
+  cat(prvec(o, width = 50, prefix = "  "), "\n",sep="")
+  outputs <- c(l$out_cmt,l$out_cap)
+  cat("- Outputs: [",length(outputs),"]", "\n",sep="")
+  cat(prvec(outputs,width = 50, prefix="  "), "\n",sep="")
   return(invisible(NULL))
 }
 
@@ -652,7 +690,8 @@ parin <- function(x) {
     maxsteps=as.integer(x@maxsteps),mxhnil=x@mxhnil,
     verbose=as.integer(x@verbose),debug=x@debug,
     digits=x@digits, tscale=x@tscale,
-    mindt=x@mindt, advan=x@advan
+    mindt=x@mindt, advan=x@advan, 
+    ss_n = 500, ss_fixed = FALSE
   )
 }
 
@@ -672,12 +711,6 @@ file_show <- function(x,spec=TRUE,source=TRUE,...) {
   do.call(base::file.show,what)
 }
 
-# re_build <- function(x,model=model(x),temp = FALSE) {
-#   if(temp) {
-#     model <- basename(tempfile(pattern="mod", tmpdir='.'))
-#   }
-#   mcode(model,x@code)
-# }
 
 ##' @export
 all.equal.mrgmod <- function(target, current,...) {
@@ -688,10 +721,3 @@ all.equal.mrgmod <- function(target, current,...) {
   t2 <- identical(target.env, current.env)
   all(t1,t2)
 }
-
-# get_model_md5 <- function(x) {
-#   files <- c(cfile(x),x@shlib[["include"]])
-#   tools::md5sum(files)
-# }
-
-
