@@ -30,8 +30,8 @@ generate_rdefs <- function(pars,
     dxdt <- paste0("dxdt_",cmt)
     init <- paste0(cmt, "_0")
 
-    cmtindex <- seq_along(cmt)-1
-    parsindex <- seq_along(pars)-1
+    cmtindex <- seq_along(cmt)-1L
+    parsindex <- seq_along(pars)-1L
 
     cmtdef <-  paste0("#define ", cmt,  " _A_[",    cmtindex,"]")
     initdef <- paste0("#define ", init, " _A_0_[",  cmtindex,"]")
@@ -226,8 +226,9 @@ touch_funs <- function(x,keep_pointers=TRUE) {
   neta <- sum(nrow(omat(x)))
   neps <- sum(nrow(smat(x)))
   
-  out <- .Call(`_mrgsolve_TOUCH_FUNS`,param,init,neta,neps,x@capture,funp,
-               x@envir)
+  out <- .Call(`_mrgsolve_TOUCH_FUNS`,param,init,
+               neta,neps,x@capture,funp,
+               x@envir, PACKAGE = "mrgsolve")
   
   names(out$init) <- names(init)
   
