@@ -1,4 +1,4 @@
-# Copyright (C) 2013 - 2019  Metrum Research Group, LLC
+# Copyright (C) 2013 - 2019  Metrum Research Group
 #
 # This file is part of mrgsolve.
 #
@@ -73,9 +73,10 @@
 ##' 
 ##' maint <- ev(time=12, cmt=1, amt=500, ii=12, addl=10)
 ##' 
-##' loading + maint
-##'
-##'
+##' c(loading, maint)
+##' 
+##' loading$time
+##' 
 ##' @export
 setGeneric("ev", function(x,...) {
   standardGeneric("ev")
@@ -97,7 +98,7 @@ setMethod("ev", "mrgmod", function(x,object=NULL,...) {
 
 ##' @rdname ev
 ##' @export
-setMethod("ev", "missing", function(time=0, amt, evid=1, cmt=1, ID=numeric(0), 
+setMethod("ev", "missing", function(time=0, amt=0, evid=1, cmt=1, ID=numeric(0), 
                                     replicate=TRUE, until=NULL, tinf=NULL,
                                     realize_addl=FALSE, ...) {
   
@@ -106,34 +107,41 @@ setMethod("ev", "missing", function(time=0, amt, evid=1, cmt=1, ID=numeric(0),
   }
   
   if(any(evid==0)) {
-    stop("evid cannot be 0 (observation)")
+    wstop("evid cannot be 0 (observation)")
   }
   
   if(missing(amt)) {
-    stop("argument \"amt\" is missing, with no default.", call.=FALSE)  
+    wstop("argument \"amt\" is missing")
   }
   
-  data <- as_tibble(list(time=time, cmt=cmt, amt=amt, evid=evid, ...))
+  l <- list(time=time, cmt=cmt, amt=amt, evid=evid)
+  if(is.numeric(tinf) && length(tinf) > 0) l[["tinf"]] <- tinf
+  if(is.numeric(until) && length(until) > 0) l[["until"]] <- until
   
-  data <- as.data.frame(data)
-  
-  if("total" %in% names(data)) {
-    data[["addl"]] <- data[["total"]]-1
-    data[["total"]] <- NULL
-  }
-  
-  if(is.numeric(tinf)) {
-    if(tinf > 0) {
-      data[["rate"]] <- data[["amt"]]/tinf  
+  qu <- quos(...)
+  if(length(qu) > 0) {
+    na1 <- names(l)
+    na2 <- names(qu)
+    j <- length(l)
+    for(i in seq_along(qu)) {
+      l[[j+i]] <- eval_tidy(qu[[i]], l)
     }
+    names(l) <- c(na1,na2)
   }
   
-  if(!missing(until)) {
-    if(!has_name("ii", data)) {
-      stop("ii is required when until is specified", call.=FALSE)
-    }
-    data[["addl"]] <- ceiling((data[["time"]] + until)/data[["ii"]])-1
+  data <- as.data.frame(as_tibble(l))
+  
+  if(all(c("rate", "tinf") %in% names(data))) {
+    wstop("input can include either rate or tinf, not both")
   }
+  if(all(c("addl", "until") %in% names(data))) {
+    wstop("input can include either addl or until, not both")
+  }
+  if(all(c("addl", "total") %in% names(data))) {
+    wstop("input can include either addl or total, not both")
+  }
+  
+  data <- finalize_ev(data)
   
   if(length(ID) > 0) {
     
@@ -153,10 +161,9 @@ setMethod("ev", "missing", function(time=0, amt, evid=1, cmt=1, ID=numeric(0),
         rownames(data) <- NULL
       } else {
         data <- data.frame(.Call(`_mrgsolve_EXPAND_EVENTS`, 
-                                 PACKAGE="mrgsolve", 
                                  match("ID", colnames(data),0), 
                                  data.matrix(data), 
-                                 ID))
+                                 ID, PACKAGE="mrgsolve" ))
       }
       
     } else {
@@ -166,13 +173,9 @@ setMethod("ev", "missing", function(time=0, amt, evid=1, cmt=1, ID=numeric(0),
       }
       data[["ID"]] <- ID
     }
-    data <- dplyr::select(data,c("ID", "time", "cmt"),everything())
-  } else {
-    data <- dplyr::select(data,c("time", "cmt"),everything())
   }
   
   if(realize_addl) data <- realize_addl(data)
-  
   return(new("ev", data=data))
 })
 
@@ -235,9 +238,9 @@ setMethod("as.ev", "data.frame", function(x,keep_id=TRUE,clean = FALSE,...) {
     x[["evid"]] <- 1 
   } else {
     x[["evid"]] <- na2zero(x[["evid"]])
-    x <- x[x$evid != 0,] 
+    x <- x[x[["evid"]] != 0,] 
     if(nrow(x)==0) {
-      stop("no dosing events found", call. = FALSE) 
+      wstop("no dosing events found; could not coerce to ev object") 
     }
   }
   
@@ -248,6 +251,9 @@ setMethod("as.ev", "data.frame", function(x,keep_id=TRUE,clean = FALSE,...) {
     keep <- intersect(keep, names(x))
     x <- x[,keep]
   }
+  
+  x <- finalize_ev(x)
+  
   new("ev", data=x)
 })
 
@@ -273,30 +279,31 @@ collect_ev <- function(...) {
   mx <- cumsum(c(0,mx[-length(mx)]))
   y <- mapply(y,mx, FUN=function(yi,mxi) return(yi+mxi), SIMPLIFY=FALSE)
   x <- bind_rows(x)
-  x <- mutate(x,ID = unlist(y,use.names=FALSE))
+  x[["ID"]] <- unlist(y,use.names=FALSE)
   tran <- intersect(tran,names(x))
   what <- names(x) %in% tran
   
   x <- mutate_at(x,which(what),list(~na2zero(.)))
   
   na.check <- which(!what)
+  
   if(length(na.check) > 0) {
     if(any(is.na(unlist(x[,na.check])))) {
-      warning("missing values in some columns.",call.=FALSE)
+      warning("missing values in some columns",call.=FALSE)
     }
   }
   x <- dplyr::select(x,c(match(tran,names(x)),seq_along(names(x))))
   
   if(!any(c("time", "TIME") %in% names(x))) {
-    stop("No time or TIME column in the data set", call. = FALSE) 
+    wstop("no time or TIME column in the data set") 
   }
   
   if(!any(c("cmt", "CMT") %in% names(x))) {
-    stop("No cmt or CMT column", call. = FALSE) 
+    wstop("no cmt or CMT column in the data set") 
   }
   
   if(!has_ID(x)) {
-    stop("No ID column in the data set", call. = FALSE)
+    wstop("no ID column in the data set")
   }
   return(x)
 }
