@@ -1,4 +1,4 @@
-# Copyright (C) 2013 - 2019  Metrum Research Group, LLC
+# Copyright (C) 2013 - 2019  Metrum Research Group
 #
 # This file is part of mrgsolve.
 #
@@ -22,6 +22,7 @@ Sys.setenv(R_TESTS="")
 options("mrgsolve_mread_quiet"=TRUE)
 
 mod <- mrgsolve:::house() %>% update(end=240)
+cap <- as.list(mod)$capture
 
 context("test-mrgsim_q")
 
@@ -29,14 +30,18 @@ data(exTheoph)
 temp <- expand.ev(amt = c(100,300,100), rate = 100, F1 = 0.2, WT = 120)
 
 test_that("simulation with a complete data set", {
-  out1 <- mrgsim_d(mod,exTheoph)
-  out2 <- mrgsim_q(mod,exTheoph)
+  out1 <- mrgsim_d(mod,exTheoph)@data
+  out2 <- mrgsim_q(mod,exTheoph)@data
   expect_identical(out1,out2)
 })
 
-test_that("simulation with a dosing data set", {
-  out1 <- mrgsim_d(mod,temp)
-  out2 <- mrgsim_q(mod,temp,stime=stime(mod),simcall=1)
+test_that("simcall=1 is deprecated", {
+  expect_error(mrgsim_q(mod,temp,stime=stime(mod),simcall=1))
+})
+
+test_that("qsim issue-490", {
+  out1 <- mrgsim_df(mod,temp)
+  out2 <- qsim(mod,temp,output="df")
   expect_identical(out1,out2)
 })
 
